import itertools
import numpy as np

O2 = (1 + 1j * 3 ** .5) / 2
O3 = (5 + 1j * 11 ** .5) / 6

CUBE = np.array(tuple(itertools.product(range(2), repeat=3)))

GENERATORS = np.array([1, O2, O3, O2 * O3])
MOSER_SPINDLE = np.array([
    [0, 0, 0, 0],
    [1, 0, 0, 0],
    [0, 1, 0, 0],
    [1, 1, 0, 0],
    [0, 0, 1, 0],
    [0, 0, 0, 1],
    [0, 0, 1, 1]
]) @ GENERATORS
MOSER_SPINDLE = np.stack([MOSER_SPINDLE.real, MOSER_SPINDLE.imag], axis=-1)

OCTAHEDRON = np.array([
    [-1, 0, 0],
    [1, 0, 0],
    [0, -1, 0],
    [0, 1, 0],
    [0, 0, -1],
    [0, 0, 1]
]) * 2 ** -.5

TETRAHEDRON = np.array([
    [1, 0, -2 ** -.5],
    [-1, 0, -2 ** -.5],
    [0, 1, 2 ** -.5],
    [0, -1, 2 ** -.5]
]) / 2


def is_infinitesimally_rigid(vertices: np.ndarray) -> bool:
    raise NotImplementedError("TODO")

def test_is_infinitesimally_rigid():
    for vertices, is_infinitesimally_rigid_true in [
        (MOSER_SPINDLE[:2], True),
        (MOSER_SPINDLE[:3], True),
        (MOSER_SPINDLE[:4], True),
        (MOSER_SPINDLE[1:], False),
        (MOSER_SPINDLE, True),
        (CUBE, False),
        (OCTAHEDRON, True),
        (TETRAHEDRON, True),
        (TETRAHEDRON[:3], True)
    ]:
        if is_infinitesimally_rigid(vertices) ^ is_infinitesimally_rigid_true:
            raise ValueError(
                f"The UDG {vertices} is {'not ' * (not is_infinitesimally_rigid_true)}infinitesimally rigid, but the `is_infinitesimally_rigid` functions claims it is {'not '* is_infinitesimally_rigid_true}so"
            )
        
    print("All tests passed. Congratulations!")


if __name__ == "__main__":
    test_is_infinitesimally_rigid()